@@ -45,10 +45,7 @@
 #include <LYLocal.h>
 
 #include <LYLeaks.h>
-
-#ifdef HAVE_SYS_WAIT_H
-#include <sys/wait.h>
-#endif
+#include <www_wait.h>
 
 struct _HTStream
 {
@@ -133,7 +130,7 @@ PUBLIC void add_lynxcgi_environment ARGS1(
 {
     char *env_value;
 
-    env_value = getenv(variable_name);
+    env_value = LYGetEnv(variable_name);
     if (env_value != NULL) {
 	char *add_value = NULL;
 
@@ -143,6 +140,16 @@ PUBLIC void add_lynxcgi_environment ARGS1(
     }
 }
 
+#ifdef __MINGW32__
+PRIVATE int LYLoadCGI ARGS4(
+	CONST char *, 		arg,
+	HTParentAnchor *,	anAnchor,
+	HTFormat,		format_out,
+	HTStream*,		sink)
+{
+	return -1;
+}
+#else
 PRIVATE int LYLoadCGI ARGS4(
 	CONST char *, 		arg,
 	HTParentAnchor *,	anAnchor,
@@ -163,7 +170,7 @@ PRIVATE int LYLoadCGI ARGS4(
     char *pgm_buff = NULL;		/* PATH_INFO extraction buffer       */
     char *path_translated;		/* From document_root/path_info      */
 
-    if (!arg || !*arg || strlen(arg) <= 8) {
+    if (isEmpty(arg) || strlen(arg) <= 8) {
 	HTAlert(BAD_REQUEST);
 	status = -2;
 	return(status);
@@ -181,7 +188,7 @@ PRIVATE int LYLoadCGI ARGS4(
     }
 
     StrAllocCopy(orig_pgm, pgm);
-    if ((cp=strchr(pgm, '#')) != NULL) {
+    if ((cp = trimPoundSelector(pgm)) != NULL) {
 	/*
 	 *  Strip a #fragment from path.  In this case any pgm_args
 	 *  found above will also be bogus, since the '?' came after
@@ -189,7 +196,6 @@ PRIVATE int LYLoadCGI ARGS4(
 	 *  handle the case where a '#' appears after a '?' properly
 	 *  according to URL rules. - kw
 	 */
-	*cp = '\0';
 	pgm_args = NULL;
     }
     HTUnEscape(pgm);
@@ -201,7 +207,7 @@ PRIVATE int LYLoadCGI ARGS4(
 	while (statrv < 0 || (statrv = stat(pgm_buff, &stat_buf)) < 0) {
 	    if ((cp=strrchr(pgm_buff, '/')) != NULL) {
 		*cp = '\0';
-		statrv = 999;	/* force new stat()  - kw */
+		statrv = 1;	/* force new stat()  - kw */
 	    } else {
 		PERROR("strrchr(pgm_buff, '/') returned NULL");
 	    	break;
@@ -322,7 +328,7 @@ PRIVATE int LYLoadCGI ARGS4(
 	HTStream *target  = NULL;		/* Unconverted data */
 	int fd1[2], fd2[2];
 	char buf[1024];
-	pid_t pid;
+	int pid;
 #ifdef HAVE_TYPE_UNIONWAIT
 	union wait wstatus;
 #else
@@ -392,16 +398,20 @@ PRIVATE int LYLoadCGI ARGS4(
 
 		if (anAnchor->post_data) {
 		    int written, remaining, total_written = 0;
+
 		    close(fd1[0]);
 
 		    /* We have form data to push across the pipe */
-		    CTRACE((tfp, "LYNXCGI: Doing post, content-type '%s'\n",
+		    if (TRACE) {
+			CTRACE((tfp, "LYNXCGI: Doing post, content-type '%s'\n",
 				anAnchor->post_content_type));
-		    CTRACE((tfp, "LYNXCGI: Writing:\n%s----------------------------------\n",
-				anAnchor->post_data));
-		    remaining = strlen(anAnchor->post_data);
+			CTRACE((tfp, "LYNXCGI: Writing:\n"));
+			trace_bstring(anAnchor->post_data);
+			CTRACE((tfp, "----------------------------------\n"));
+		    }
+		    remaining = BStrLen(anAnchor->post_data);
 		    while ((written = write(fd1[1],
-					    anAnchor->post_data + total_written,
+					    BStrData(anAnchor->post_data) + total_written,
 					    remaining)) != 0) {
 			if (written < 0) {
 #ifdef EINTR
@@ -518,7 +528,7 @@ PRIVATE int LYLoadCGI ARGS4(
 		    add_environment_value("REQUEST_METHOD=POST");
 
 		    HTSprintf0(&post_len, "CONTENT_LENGTH=%d",
-			    strlen(anAnchor->post_data));
+			       BStrLen(anAnchor->post_data));
 		    add_environment_value(post_len);
 		} else {
 		    close(fileno(stdin));
@@ -650,7 +660,8 @@ PRIVATE int LYLoadCGI ARGS4(
 			   format_out,
 			   sink, anAnchor);
 
-    HTSprintf0(&buf, "<head>\n<title>%s</title>\n</head>\n<body>\n", gettext("Good Advice"));
+    HTSprintf0(&buf, "<html>\n<head>\n<title>%s</title>\n</head>\n<body>\n",
+	       gettext("Good Advice"));
     (*target->isa->put_block)(target, buf, strlen(buf));
 
     HTSprintf0(&buf, "<h1>%s</h1>\n", gettext("Good Advice"));
@@ -670,7 +681,7 @@ PRIVATE int LYLoadCGI ARGS4(
 	       gettext("It provides state of the art CGI script support.\n"));
     (*target->isa->put_block)(target, buf, strlen(buf));
 
-    HTSprintf0(&buf,"</body>\n");
+    HTSprintf0(&buf,"</body>\n</html>\n");
     (*target->isa->put_block)(target, buf, strlen(buf));
 
     (*target->isa->_free)(target);
@@ -683,6 +694,7 @@ PRIVATE int LYLoadCGI ARGS4(
 #endif /* LYNXCGI_LINKS */
     return(status);
 }
+#endif /* __MINGW32__ */
 
 #ifdef GLOBALDEF_IS_MACRO
 #define _LYCGI_C_GLOBALDEF_1_INIT { "lynxcgi", LYLoadCGI, 0 }
