@@ -18,6 +18,7 @@
 
 #define DONT_USE_SOCKS5
 #include <HTUtils.h>
+
 /*
  *  Don't try to use LYexit().
  */
@@ -25,14 +26,6 @@
 #undef exit
 #endif /* exit */
 
-#ifndef TOUPPER
-#define TOUPPER(c) (islower((unsigned char)c) ? toupper((unsigned char)c) : (c))
-#endif /* !TOLOWER */
-
-#ifndef TOLOWER
-#define TOLOWER(c) (isupper((unsigned char)c) ? tolower((unsigned char)c) : (c))
-#endif /* !TOLOWER */
-
 #include <UCkd.h>
 #include <UCDefs.h>
 
@@ -79,6 +72,16 @@ PRIVATE void usage NOARGS
     done(EX_USAGE);
 }
 
+#ifdef EXP_ASCII_CTYPES
+PUBLIC int ascii_tolower ARGS1(int, i)
+{
+    if ( 91 > i && i > 64 )
+	return (i+32);
+    else
+	return i;
+}
+#endif
+
 /* copied from HTString.c, not everybody has strncasecmp */
 PUBLIC int strncasecomp ARGS3(
 	CONST char*,	a,
@@ -112,8 +115,11 @@ PRIVATE int getunicode ARGS1(
     if (*p == '-') {
 	return -2;
     } else if (*p != 'U' || p[1] != '+' ||
-	       !isxdigit(p[2]) || !isxdigit(p[3]) || !isxdigit(p[4]) ||
-	       !isxdigit(p[5]) || isxdigit(p[6])) {
+	       !isxdigit(UCH(p[2])) ||
+	       !isxdigit(UCH(p[3])) ||
+	       !isxdigit(UCH(p[4])) ||
+	       !isxdigit(UCH(p[5])) ||
+	        isxdigit(UCH(p[6]))) {
 	return -1;
     }
     *p0 = p+6;
@@ -431,7 +437,7 @@ PUBLIC int main ARGS2(
 		while (*p == ' ' || *p == '\t') {
 		    p++;
 		}
-		useDefaultMap = (*p == '1' || tolower(*p) == 'y');
+		useDefaultMap = (*p == '1' || TOLOWER(*p) == 'y');
 		continue;
 
 	    case 'M':
@@ -559,9 +565,9 @@ PUBLIC int main ARGS2(
 		 *  We had ':'.
 		 */
 		for (ch = *(++p); (ch = *p) != '\0'; p++, p1++) {
-		    if ((unsigned char)ch < 32 || ch == '\\' || ch == '\"' ||
-			(unsigned char)ch >= 127) {
-			sprintf(p1, "\\%.3o", (unsigned char)ch);
+		    if (UCH(ch) < 32 || ch == '\\' || ch == '\"' ||
+			UCH(ch) >= 127) {
+			sprintf(p1, "\\%.3o", UCH(ch));
 #ifdef NOTDEFINED
 			fprintf(stderr, "%s\n", tbuf);
 #endif /* NOTDEFINED */
@@ -763,7 +769,7 @@ PUBLIC int main ARGS2(
 	for (i = 0, p = this_MIMEcharset;
 	     *p && (i < UC_MAXLEN_ID_APPEND-1);
 	     p++, i++) {
-	    id_append[i+1] = isalnum(*p) ? *p : '_';
+	    id_append[i+1] = isalnum(UCH(*p)) ? *p : '_';
 	}
 	id_append[i+1] = '\0';
     }
@@ -881,4 +887,5 @@ id_append, id_append, nuni, id_append, lowest_eight, RawOrEnc, CodePage);
     }
 
     done(EX_OK);
+    return 0;
 }
